/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

#include <locale.h>
#include <config.h>
#include <i18n-support.h>
#include <gnome.h>

#include "gstmediaplay.h"
#include "gstvideowidget.h"
#include "gstpreferences.h"
#include "gstmediainfo.h"
#include "gstcontrol.h"
#include "gstplaylist.h"

static void 	gst_media_play_class_init      	 (GstMediaPlayClass *klass);
static void 	gst_media_play_init       	 (GstMediaPlay *play);

/* signal handlers for GstControl */
static void 	gst_media_play_slider_changed    (GtkAdjustment *adj, GstMediaPlay *mplay);
static void     gst_media_play_toggle_mute       (GtkWidget *widget, GstMediaPlay *mplay);
static void     gst_media_play_seek_to_pos       (GtkWidget *widget, gdouble value, GstMediaPlay *mplay);
static void     gst_media_play_toggle_play_pause (GtkWidget *widget, GstMediaPlay *mplay);
static void     gst_media_play_stop              (GtkWidget *widget, GstMediaPlay *mplay);
static void     gst_media_play_show_playlist     (GtkWidget *widget, GstMediaPlay *mplay);
static void	gst_media_play_display_mode_change (GtkWidget *widget, GstMediaPlayMode display_mode, GstMediaPlay *mplay);
static void	gst_media_play_toggle_display_mode (GtkWidget *widget, GstMediaPlay *mplay);
static void	gst_media_play_fullscreen	 (GtkWidget *widget, GstMediaPlay *mplay);
static void	gst_media_play_volume_changed    (GtkWidget *widget, gdouble value, GstMediaPlay *mplay);

/* signal handlers for playlist */
static void     gst_media_play_play_next         (GtkWidget *widget, GstMediaPlay *mplay);
static void     gst_media_play_set_selected      (GtkWidget *widget, GstMediaPlay *mplay);

/* signal handlers for GstPlay */
static void     gst_media_play_time_tick         (GstPlay *play, gint64 time_nanos, GstMediaPlay *mplay);
static void     gst_media_play_got_length        (GstPlay *play, gint64 length_nanos, GstMediaPlay *mplay);
static void	gst_media_play_have_xid          (GstPlay *play, gint xid, GstMediaPlay *mplay);
static void	gst_media_play_have_video_size   (GstPlay *play, gint width, gint height, GstMediaPlay *mplay);
static void	gst_media_play_state_change      (GstPlay *play, GstElementState old_state, GstElementState new_state, GstMediaPlay *mplay);

static GObject *parent_class = NULL;

enum {
	STATE_CHANGE,
	DISPLAY_MODE_CHANGE,
	LAST_SIGNAL
};

GType
gst_media_play_get_type (void)
{
	static GType play_type = 0;
  
	if (!play_type)
	{
		static const GTypeInfo style_info =
			{
				sizeof (GstMediaPlayClass),
				(GBaseInitFunc) NULL,
				(GBaseFinalizeFunc) NULL,
				(GClassInitFunc) gst_media_play_class_init,
				NULL,           /* class_finalize */
				NULL,           /* class_data */
				sizeof (GstMediaPlay),
				0,              /* n_preallocs */
				(GInstanceInitFunc) gst_media_play_init,
			};
		
		play_type = g_type_register_static (GTK_TYPE_VBOX,
						    "GstMediaPlay",
						    &style_info, 0);
	}
	
	return play_type;
}
static gint gst_media_play_signals [LAST_SIGNAL] = { 0 };

extern void _gstplay_stock_icons_init (void);

static void
gst_media_play_class_init (GstMediaPlayClass *klass)
{
	GtkWidgetClass  *widget_class = (GtkWidgetClass *) klass;
	GtkObjectClass  *object_class = (GtkObjectClass *) klass;
	GObjectClass    *gobject_class = (GObjectClass *) klass;

	/* delete me? */
	_gstplay_stock_icons_init ();

	gst_media_play_signals [STATE_CHANGE] = 
		g_signal_new ("state_change", 
			      G_TYPE_FROM_CLASS (klass), 
			      G_SIGNAL_RUN_FIRST,
			      G_STRUCT_OFFSET (GstMediaPlayClass, state_changed), 
			      NULL, NULL,
			      gst_marshal_VOID__INT_INT,
			      G_TYPE_NONE, 2, 
			      G_TYPE_INT, G_TYPE_INT);
	gst_media_play_signals [DISPLAY_MODE_CHANGE] =
		g_signal_new ("display_mode_change",
			      G_TYPE_FROM_CLASS (gobject_class),
			      G_SIGNAL_RUN_FIRST,
			      G_STRUCT_OFFSET (GstMediaPlayClass, display_mode_change),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__INT,
			      G_TYPE_NONE, 1,
			      G_TYPE_INT);
}
static void
gst_media_play_init (GstMediaPlay *mplay)
{
	GtkWidget         *playlist;

	/* play object */
	mplay->play = gst_play_new (GST_PLAY_PIPE_VIDEO);

	g_return_if_fail(mplay->play != NULL);

	g_signal_connect (G_OBJECT (mplay->play), "stream_end",
			  G_CALLBACK (gst_media_play_play_next), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "time_tick", 
			  G_CALLBACK (gst_media_play_time_tick), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "stream_length", 
			  G_CALLBACK (gst_media_play_got_length), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "have_xid", 
			  G_CALLBACK (gst_media_play_have_xid), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "have_video_size", 
			  G_CALLBACK (gst_media_play_have_video_size), mplay);
	g_signal_connect (G_OBJECT (mplay->play), "state_change", 
			  G_CALLBACK (gst_media_play_state_change), mplay);
	
	gtk_box_set_homogeneous (GTK_BOX(mplay), FALSE);
	gtk_box_set_spacing (GTK_BOX(mplay), 0);
	/* player widgets */

	mplay->video_widget = gst_video_widget_new ();
	g_return_if_fail(mplay->video_widget != NULL);

	mplay->control = gst_control_new ();
	g_return_if_fail(mplay->control != NULL);
	
	gtk_box_pack_start (GTK_BOX (mplay), mplay->video_widget, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (mplay), GTK_WIDGET (mplay->control), FALSE, FALSE, 0);

	g_signal_connect (G_OBJECT (mplay->control), "button_play",
			  G_CALLBACK (gst_media_play_toggle_play_pause), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "button_stop",
			  G_CALLBACK (gst_media_play_stop), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "button_list",
			  G_CALLBACK (gst_media_play_show_playlist), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "button_mute",
			  G_CALLBACK (gst_media_play_toggle_mute), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "volume_change",
			  G_CALLBACK (gst_media_play_volume_changed), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "seek_change",
			  G_CALLBACK (gst_media_play_seek_to_pos), mplay);
	g_signal_connect (G_OBJECT (mplay->control), "display_mode_change",
			  G_CALLBACK (gst_media_play_display_mode_change), mplay);
	
	mplay->display_mode = GST_MEDIA_PLAY_NORMAL;

	mplay->playlist = gst_playlist_new ();
	g_return_if_fail(mplay->playlist != NULL);

	gtk_box_pack_start (GTK_BOX (mplay), GTK_WIDGET (mplay->playlist), TRUE, TRUE, 0);
	g_signal_connect (G_OBJECT (mplay->playlist), "selection_change",
			  G_CALLBACK (gst_media_play_set_selected), mplay);

	gtk_widget_show (mplay->control);

	mplay->uri = NULL;
}

GstMediaPlay*
gst_media_play_new (void)
{
	return GST_MEDIA_PLAY (g_object_new (GST_TYPE_MEDIA_PLAY, NULL));
}
  


void
gst_media_play_set_state (GstMediaPlay *mplay, GstElementState state)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_play_set_state (mplay->play, state);
}

static void
gst_media_play_toggle_play_pause (GtkWidget *widget, GstMediaPlay *mplay)
{
	GstElementState playstate;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	playstate = gst_play_get_state (mplay->play);

	if (playstate == GST_STATE_PLAYING) {
		gst_media_play_set_state (mplay, GST_STATE_PAUSED);
	}
	else {
		gst_media_play_set_state (mplay, GST_STATE_PLAYING);
	}
}

static void
gst_media_play_stop (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	gst_media_play_set_state (mplay, GST_STATE_READY);
}

static void
gst_media_play_volume_changed (GtkWidget *widget, gdouble value, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_play_set_volume (mplay->play, value);
}

void
gst_media_play_set_volume (GstMediaPlay *mplay, gdouble value)
{
	GtkAdjustment *adj;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	adj = gst_control_get_volume_adjustment(GST_CONTROL(mplay->control));
	
	value = value > 1.0 ? 1.0 : value;
	value = value < 0.0 ? 0.0 : value;
	gtk_adjustment_set_value(adj, value);
}

gdouble
gst_media_play_get_volume (GstMediaPlay *mplay)
{
	GtkAdjustment *adj;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	adj = gst_control_get_volume_adjustment(GST_CONTROL(mplay->control));
	return gtk_adjustment_get_value(adj);
}

void
gst_media_play_load_uri (GstMediaPlay *mplay, const gchar *uri)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	mplay->uri = uri;
	
	/* FIXME send a signal 
	gtk_window_set_title (GTK_WINDOW (mplay->window), g_strdup_printf ("%s", uri));*/

	/*g_print ("loading %s\n", uri);*/
	gst_play_set_location (mplay->play, uri);
}

static void
gst_media_play_play_next (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_next_uri(mplay);
}

static void
gst_media_play_set_selected (GtkWidget *widget, GstMediaPlay *mplay)
{
	gchar *uri;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	uri = gst_playlist_get_selected (GST_PLAYLIST (mplay->playlist));
	gst_media_play_load_uri (mplay, g_strdup_printf ("%s", uri));
}

void
gst_media_play_add_uri(GstMediaPlay *mplay, const gchar *uri)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	gst_playlist_append (GST_PLAYLIST (mplay->playlist), uri);

	if (mplay->uri == NULL) {
		gst_playlist_set_selected (GST_PLAYLIST (mplay->playlist), "0");
	}
}

void
gst_media_play_clear_uri_list(GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_playlist_clear (GST_PLAYLIST (mplay->playlist));
}

void
gst_media_play_next_uri(GstMediaPlay *mplay)
{
	gboolean  ret;
	gchar     *uri;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	ret = gst_playlist_get_next (GST_PLAYLIST (mplay->playlist), &uri);

	if (ret != FALSE) {
		gst_media_play_load_uri (mplay, g_strdup_printf ("%s", uri));
	}
}

void
gst_media_play_prev_uri(GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

}

static void
gst_media_play_toggle_mute (GtkWidget *widget, GstMediaPlay *mplay)
{
	gboolean muted;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	muted = !gst_play_get_mute(mplay->play);
	gst_play_set_mute(mplay->play, muted);
	gst_control_set_mute(GST_CONTROL(mplay->control), muted);
}

static void
gst_media_play_seek_to_pos (GtkWidget *widget, gdouble value, GstMediaPlay *mplay)
{
	gdouble length_nanos;
	gint64 seek_time;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	length_nanos = (gdouble)mplay->length_nanos;
	seek_time = (gint64)(length_nanos * value);
	gst_play_seek_to_time (mplay->play, seek_time);
}

void
gst_media_play_set_video_scale (GstMediaPlay *mplay, gfloat scale_factor)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	if (mplay->video_widget) {
		g_object_set (G_OBJECT (mplay->video_widget), "scale_factor", scale_factor, NULL);
	}
}

void
gst_media_play_show_media_info (GstMediaPlay *mplay)
{
	GtkWidget *media_info;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	if (mplay->uri != NULL) {
		media_info = gst_media_info_new (mplay->uri);
	}
	else {
		media_info = gst_media_info_new ("no file loaded");
	}

	gtk_widget_show (media_info);
}

void
gst_media_play_show_preferences (GstMediaPlay *mplay)
{
	GtkWidget *preferences;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	preferences = gst_preferences_new ();
	
	gtk_widget_show (preferences);
}

void
gst_media_play_show_about (GstMediaPlay *mplay)
{
	static GtkWidget *about;

	const gchar *authors[] = {
		"Steve Baker <stevebaker_org@yahoo.co.uk>", 
		"Richard Boulton <richard@tartarus.org>",
		"Ronald Bultje <rbultje@ronald.bitfreak.net>",
		"Arik Devens <arik@gnome.org>",
		"Chris Emerson (PPC port)",
		"Benjamin Otte <in7y118@public.uni-hamburg.de>",
		"Wim Taymans <wim.taymans@tvd.be>",
		"Erik Walthinsen <omega@cse.ogi.edu>", 
		"Thomas Vander Stichele <thomas@apestaart.org>", 
		NULL};

	if (about != NULL) 
	{
                gdk_window_raise (about->window);
                gdk_window_show (about->window);
                return;
        }

	about = gnome_about_new ("GStreamer Player", GST_PLAYER_VERSION, 
				 "(C) 1999-2002 The GStreamer Team",
				 "A generic media player for the GStreamer media framework.",
				 (const char **)authors,
				 NULL,
				 NULL,
				 NULL);

	g_signal_connect (G_OBJECT (about), "destroy",
			  G_CALLBACK (gtk_widget_destroyed), &about);
	
        gtk_widget_show (about);
}

static void
gst_media_play_show_playlist (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	if (GTK_WIDGET_VISIBLE (mplay->playlist) == TRUE)
	{
		gtk_widget_hide (mplay->playlist);
	}
	else
	{
		gtk_widget_show (mplay->playlist);
	}
}

void
gst_media_play_fullscreen (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_set_display_mode (mplay, GST_MEDIA_PLAY_FULLSCREEN);
}

void
gst_media_play_toggle_display_mode (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	if (mplay->display_mode == GST_MEDIA_PLAY_NORMAL){
		gst_media_play_set_display_mode(mplay, GST_MEDIA_PLAY_MINI);
	}
	else if (mplay->display_mode == GST_MEDIA_PLAY_MINI ||
	         mplay->display_mode == GST_MEDIA_PLAY_FULLSCREEN){
		gst_media_play_set_display_mode(mplay, GST_MEDIA_PLAY_NORMAL);
	}
}

static void
gst_media_play_display_mode_change (GtkWidget *widget, GstMediaPlayMode display_mode, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_set_display_mode(mplay, display_mode);
}

static void
gst_media_play_set_fullscreen (GstMediaPlay *mplay, gboolean fullscreen)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
  
	if (fullscreen){
		GdkRectangle screen;

		/* return if we're in the right state */
		if (mplay->display_mode == GST_MEDIA_PLAY_FULLSCREEN){
			return;
		}
		

		/* get position of screen */
		/* FIXME: make me work with Xinerama, please.
		 * see galeon/.../window.c for an example */
		screen.x = 0;
		screen.y = 0;
		screen.height = gdk_screen_height();
		screen.width = gdk_screen_width();

		/* create new fullscreen widget */
		mplay->window_fs = GTK_WINDOW (gtk_window_new (GTK_WINDOW_POPUP));
		if (mplay->window_fs == NULL)
		{
			g_warning ("failed to create window for fullscreen\n");
			return;
		}
		gtk_window_move (mplay->window_fs, screen.x, screen.y);
		gtk_window_resize (mplay->window_fs, screen.width, screen.height);

		mplay->fs_vbox = gtk_vbox_new (FALSE, 0);
		gtk_container_add (GTK_CONTAINER (mplay->window_fs), mplay->fs_vbox);
		gtk_grab_add (GTK_WIDGET (mplay->window_fs));

		g_object_ref(mplay->video_widget);
		gst_video_widget_reset_socket(GST_VIDEO_WIDGET(mplay->video_widget));
		gtk_container_remove(GTK_CONTAINER(mplay), mplay->video_widget);
		gtk_box_pack_start (GTK_BOX (mplay->fs_vbox), mplay->video_widget, TRUE, TRUE, 0);
		g_object_unref(mplay->video_widget);

		gst_play_need_new_video_window(mplay->play);

		g_object_ref(G_OBJECT(mplay->control));
		gtk_container_remove(GTK_CONTAINER(mplay), mplay->control);
		gtk_box_pack_start (GTK_BOX (mplay->fs_vbox), mplay->control, FALSE, FALSE, 0);
		g_object_unref(mplay->control);

		gtk_widget_show (mplay->fs_vbox);
		gtk_widget_show (GTK_WIDGET (mplay->window_fs));

	}
	else {
		gtk_grab_remove (GTK_WIDGET (mplay->window_fs));
		gtk_widget_hide (GTK_WIDGET (mplay->window_fs));

		g_object_ref(mplay->video_widget);
		gst_video_widget_reset_socket(GST_VIDEO_WIDGET(mplay->video_widget));
		gtk_container_remove(GTK_CONTAINER(mplay->fs_vbox), mplay->video_widget);
		gtk_box_pack_start (GTK_BOX (mplay), mplay->video_widget, TRUE, TRUE, 0);
		g_object_unref(mplay->video_widget);

		gst_play_need_new_video_window(mplay->play);

		g_object_ref(G_OBJECT(mplay->control));
		gtk_container_remove(GTK_CONTAINER(mplay->fs_vbox), mplay->control);
		gtk_box_pack_start (GTK_BOX (mplay), mplay->control, FALSE, FALSE, 0);
		g_object_unref(mplay->control);

		gtk_box_reorder_child(GTK_BOX (mplay), mplay->video_widget, 0);
		gtk_box_reorder_child(GTK_BOX (mplay), mplay->control, 1);

		gtk_widget_show (mplay->control);

		gtk_widget_destroy (GTK_WIDGET (mplay->window_fs));
		mplay->window_fs = NULL;
		mplay->fs_vbox = NULL;
	}
}

void
gst_media_play_set_display_mode (GstMediaPlay *mplay, GstMediaPlayMode display_mode)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	switch (display_mode){
	case GST_MEDIA_PLAY_FULLSCREEN:
		gst_media_play_set_fullscreen (mplay, TRUE);
		break;
	case GST_MEDIA_PLAY_NORMAL:
		if (mplay->display_mode == GST_MEDIA_PLAY_FULLSCREEN){
			gst_media_play_set_fullscreen (mplay, FALSE);
		}
		break;
	case GST_MEDIA_PLAY_MINI:
		gtk_widget_hide (mplay->playlist);
		break;
	default:
		g_warning("unknown mode %d\n", display_mode);
		break;
	}
	mplay->display_mode = display_mode;
	g_signal_emit (mplay, gst_media_play_signals [DISPLAY_MODE_CHANGE], 0, display_mode);
	gst_control_set_display_mode(GST_CONTROL(mplay->control), display_mode);
}

gfloat
gst_media_play_get_video_scale (GstMediaPlay *mplay)
{
	gfloat scale_factor = 1.0;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	if (mplay->video_widget) {
		g_object_get (G_OBJECT (mplay->video_widget), "scale_factor", &scale_factor, NULL);
	}
	return scale_factor;
}

GstElementState
gst_media_play_get_state (GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	return gst_play_get_state (mplay->play);
}

GstMediaPlayMode
gst_media_play_get_display_mode (GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	return mplay->display_mode;
}

static void
gst_media_play_time_tick (GstPlay *play, gint64 time_nanos, GstMediaPlay *mplay)
{
	gdouble seek_pos;
	gint seconds;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	seek_pos = ((gdouble) time_nanos) / ((gdouble) mplay->length_nanos);
	seconds = (gint) (time_nanos / GST_SECOND);
	gst_control_set_seek_pos (mplay->control, seek_pos);
}

static void
gst_media_play_got_length (GstPlay *play, gint64 length_nanos, GstMediaPlay *mplay)
{
	gint seconds;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	seconds = (gint) (length_nanos / GST_SECOND);
	mplay->length_nanos = length_nanos;
	gst_control_set_enable_seek (GST_CONTROL (mplay->control), TRUE);
}

static void
gst_media_play_state_change (GstPlay *play, GstElementState old_state, GstElementState new_state, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	gst_control_set_state (GST_CONTROL (mplay->control), new_state);
}


static void
gst_media_play_have_xid (GstPlay *play, gint xid, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));


	if (mplay->video_widget)
	{
		gtk_widget_show (mplay->video_widget);
		gst_video_widget_set_xembed_xid(GST_VIDEO_WIDGET(mplay->video_widget), xid);
		gst_control_allow_fullscreen (GST_CONTROL(mplay->control), TRUE);
	}

}

static void
gst_media_play_have_video_size (GstPlay *play, gint width, gint height, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	if (mplay->video_widget)
	{
		gst_video_widget_set_source_size(GST_VIDEO_WIDGET(mplay->video_widget), width, height);
	}
}

